@@ -58,6 +58,7 @@
 #include "MagickCore/blob-private.h"
 #include "MagickCore/exception.h"
 #include "MagickCore/exception-private.h"
+#include "MagickCore/image-private.h"
 #include "MagickCore/memory_.h"
 #include "MagickCore/memory-private.h"
 #include "MagickCore/policy.h"
@@ -174,14 +175,14 @@ static size_t
   virtual_anonymous_memory = 0;
 
 #if defined _MSC_VER
-static void* MSCMalloc(size_t size)
+static void *MSCMalloc(size_t size)
 {
-  return malloc(size);
+  return(malloc(size));
 }
 
-static void* MSCRealloc(void* ptr, size_t size)
+static void *MSCRealloc(void* ptr, size_t size)
 {
-  return realloc(ptr,size);
+  return(realloc(ptr,size));
 }
 
 static void MSCFree(void* ptr)
@@ -247,7 +248,7 @@ static MagickBooleanType
 %    o quantum: the size (in bytes) of each object.
 %
 */
-#if MAGICKCORE_HAVE_STDC_ALIGNED_ALLOC
+#if defined(MAGICKCORE_HAVE_ALIGNED_MALLOC)
 #define AcquireAlignedMemory_Actual AcquireAlignedMemory_STDC
 static inline void *AcquireAlignedMemory_STDC(const size_t size)
 {
@@ -319,7 +320,7 @@ static inline void *AcquireAlignedMemory_Generic(const size_t size)
   #if SIZE_MAX < ALIGNMENT_OVERHEAD
     #error "CACHE_LINE_SIZE is way too big."
   #endif
-  extent=(size + ALIGNMENT_OVERHEAD);
+  extent=(size+ALIGNMENT_OVERHEAD);
   if (extent <= size)
     {
       errno=ENOMEM;
@@ -340,7 +341,10 @@ MagickExport void *AcquireAlignedMemory(const size_t count,const size_t quantum)
     size;
 
   if (HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse)
-    return(NULL);
+    {
+      errno=ENOMEM;
+      return(NULL);
+    }
   if (memory_methods.acquire_aligned_memory_handler != (AcquireAlignedMemoryHandler) NULL)
     return(memory_methods.acquire_aligned_memory_handler(size,CACHE_LINE_SIZE));
   return(AcquireAlignedMemory_Actual(size));
@@ -636,8 +640,12 @@ MagickExport void *AcquireQuantumMemory(const size_t count,const size_t quantum)
   size_t
     size;
 
-  if (HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse)
-    return((void *) NULL);
+  if ((HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse) ||
+      (size > GetMaxMemoryRequest()))
+    {
+      errno=ENOMEM;
+      return(NULL);
+    }
   return(AcquireMagickMemory(size));
 }
 
@@ -680,7 +688,10 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
     size;
 
   if (HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse)
-    return((MemoryInfo *) NULL);
+    {
+      errno=ENOMEM;
+      return((MemoryInfo *) NULL);
+    }
   if (virtual_anonymous_memory == 0)
     {
       virtual_anonymous_memory=1;
@@ -703,8 +714,7 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
   (void) memset(memory_info,0,sizeof(*memory_info));
   memory_info->length=size;
   memory_info->signature=MagickCoreSignature;
-  if ((virtual_anonymous_memory == 1) &&
-      (size <= GetMaxMemoryRequest()))
+  if ((virtual_anonymous_memory == 1) && (size <= GetMaxMemoryRequest()))
     {
       memory_info->blob=AcquireAlignedMemory(1,size);
       if (memory_info->blob != NULL)
@@ -741,7 +751,7 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
 #if !defined(MAGICKCORE_HAVE_POSIX_FALLOCATE)
                   memory_info->blob=MapBlob(file,IOMode,0,size);
 #else
-                  if (posix_fallocate(file,0,size) == 0)
+                  if (posix_fallocate(file,0,(MagickOffsetType) size) == 0)
                     memory_info->blob=MapBlob(file,IOMode,0,size);
 #endif
                   if (memory_info->blob != NULL)
@@ -1006,22 +1016,24 @@ MagickExport void GetMagickMemoryMethods(
 */
 MagickExport size_t GetMaxMemoryRequest(void)
 {
+#define MinMemoryRequest "16MiB"
+
   if (max_memory_request == 0)
     {
       char
         *value;
 
+      max_memory_request=(size_t) MagickULLConstant(~0);
       value=GetPolicyValue("system:max-memory-request");
       if (value != (char *) NULL)
         {
           /*
             The security policy sets a max memory request limit.
           */
-          max_memory_request=StringToSizeType(value,100.0);
+          max_memory_request=MagickMax(StringToSizeType(value,100.0),
+            StringToSizeType(MinMemoryRequest,100.0));
           value=DestroyString(value);
         }
-      else
-        max_memory_request=(size_t) MagickULLConstant(~0);
     }
   return(max_memory_request);
 }
@@ -1060,75 +1072,6 @@ MagickExport void *GetVirtualMemoryBlob(const MemoryInfo *memory_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   H e a p O v e r f l o w S a n i t y C h e c k                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  HeapOverflowSanityCheck() returns MagickFalse if the heap allocation request
-%  is not zero and fits within the maximum limits of size_t. Otherwise, it
-%  returns a value other than MagickFalse; if the heap allocation request does
-%  not fit within the maximum limts of size_t, then errno is set to ENOMEM.
-%
-%  The format of the HeapOverflowSanityCheck method is:
-%
-%      MagickBooleanType HeapOverflowSanityCheck(const size_t count,
-%        const size_t quantum)
-%
-%  A description of each parameter follows:
-%
-%    o count: the number of objects to allocate contiguously.
-%
-%    o quantum: the size (in bytes) of each object.
-%
-*/
-extern MagickExport MagickBooleanType HeapOverflowSanityCheck(
-  const size_t count,const size_t quantum);
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   H e a p O v e r f l o w S a n i t y C h e c k G e t S i z e               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  HeapOverflowSanityCheckGetSize() returns MagickFalse if the heap allocation
-%  request is not zero and fits within the maximum limits of size_t; in this
-%  case, the value (count*quantum) is returned in the size_t object to which
-%  points the parameter named "size".
-%
-%  Otherwise, it returns a value other than MagickFalse; if the heap allocation
-%  request does not fit within the maximum limits of size_t, then errno is set
-%  to ENOMEM. In this case, the object referenced by "size" is not modified.
-%
-%
-%  The format of the HeapOverflowSanityCheckGetSize method is:
-%
-%      MagickBooleanType HeapOverflowSanityCheckGetSize(const size_t count,
-%        const size_t quantum,size_t *const size)
-%
-%  A description of each parameter follows:
-%
-%    o count: the number of objects to allocate contiguously.
-%
-%    o quantum: the size (in bytes) of each object.
-%
-%    o size: a pointer to an object to hold (count*quantum) if all goes well.
-%
-*/
-extern MagickExport MagickBooleanType HeapOverflowSanityCheckGetSize(
-  const size_t count,const size_t quantum,size_t *const size);
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   R e l i n q u i s h A l i g n e d M e m o r y                             %
 %                                                                             %
 %                                                                             %
@@ -1156,7 +1099,7 @@ MagickExport void *RelinquishAlignedMemory(void *memory)
       memory_methods.relinquish_aligned_memory_handler(memory);
       return(NULL);
     }
-#if MAGICKCORE_HAVE_STDC_ALIGNED_ALLOC || defined(MAGICKCORE_HAVE_POSIX_MEMALIGN)
+#if defined(MAGICKCORE_HAVE_ALIGNED_MALLOC) || defined(MAGICKCORE_HAVE_POSIX_MEMALIGN)
   free(memory);
 #elif defined(MAGICKCORE_HAVE__ALIGNED_MALLOC)
   _aligned_free(memory);
@@ -1480,10 +1423,12 @@ MagickExport void *ResizeQuantumMemory(void *memory,const size_t count,
   size_t
     size;
 
-  if (HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse)
+  if ((HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse) ||
+      (size > GetMaxMemoryRequest()))
     {
+      errno=ENOMEM;
       memory=RelinquishMagickMemory(memory);
-      return((void *) NULL);
+      return(NULL);
     }
   return(ResizeMagickMemory(memory,size));
 }
