@@ -1083,11 +1083,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             if (IsStringFalse(artifact) != MagickFalse)
               highres=MagickFalse;
 #endif
-            if (highres != MagickFalse)
-              {
-                source_info.scale=1.0;
-                source_info.translate=0.0;
-              }
+            source_info.scale=1.0;
+            source_info.translate=0.0;
             source_info.colorspace=sRGBColorspace;
             source_info.channels=3;
             switch (cmsGetColorSpace(source_info.profile))
@@ -1183,11 +1180,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             signature=cmsGetPCS(source_info.profile);
             if (target_info.profile != (cmsHPROFILE) NULL)
               signature=cmsGetColorSpace(target_info.profile);
-            if (highres != MagickFalse)
-              {
-                target_info.scale=1.0;
-                target_info.translate=0.0;
-              }
+            target_info.scale=1.0;
+            target_info.translate=0.0;
             target_info.channels=3;
             switch (signature)
             {
@@ -1846,8 +1840,67 @@ static void GetProfilesFromResourceBlock(Image *image,
   }
 }
 
+static void PatchCorruptProfile(const char *name,StringInfo *profile)
+{
+  register unsigned char
+    *p;
+
+  size_t
+    length;
+
+  /*
+    Detect corrupt profiles and if discovered, repair.
+  */
+  if (LocaleCompare(name,"xmp") == 0)
+    {
+      /*
+        Remove garbage after xpacket end.
+      */
+      p=GetStringInfoDatum(profile);
+      p=(unsigned char *) strstr((const char *) p,"<?xpacket end=\"w\"?>");
+      if (p != (unsigned char *) NULL)
+        {
+          p+=19;
+          length=p-GetStringInfoDatum(profile);
+          if (length != GetStringInfoLength(profile))
+            {
+              *p='\0';
+              SetStringInfoLength(profile,length);
+            }
+        }
+      return;
+    }
+  if (LocaleCompare(name,"exif") == 0)
+    {
+      /*
+        Check if profile starts with byte order marker instead of Exif.
+      */
+      p=GetStringInfoDatum(profile);
+      if ((LocaleNCompare((const char *) p,"MM",2) == 0) ||
+          (LocaleNCompare((const char *) p,"II",2) == 0))
+        {
+          const unsigned char
+            profile_start[] = "Exif\0\0";
+
+          StringInfo
+            *exif_profile;
+
+          exif_profile=AcquireStringInfo(6);
+          if (exif_profile != (StringInfo *) NULL)
+            {
+              SetStringInfoDatum(exif_profile,profile_start);
+              ConcatenateStringInfo(exif_profile,profile);
+              SetStringInfoLength(profile,GetStringInfoLength(exif_profile));
+              SetStringInfo(profile,exif_profile);
+              exif_profile=DestroyStringInfo(exif_profile);
+            }
+        }
+    }
+}
+
 #if defined(MAGICKCORE_XML_DELEGATE)
-static MagickBooleanType ValidateXMPProfile(const StringInfo *profile)
+static MagickBooleanType ValidateXMPProfile(Image *image,
+  const StringInfo *profile,ExceptionInfo *exception)
 {
   xmlDocPtr
     document;
@@ -1859,13 +1912,20 @@ static MagickBooleanType ValidateXMPProfile(const StringInfo *profile)
     GetStringInfoLength(profile),"xmp.xml",NULL,XML_PARSE_NOERROR |
     XML_PARSE_NOWARNING);
   if (document == (xmlDocPtr) NULL)
-    return(MagickFalse);
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),ImageWarning,
+        "CorruptImageProfile","`%s' (XMP)",image->filename);
+      return(MagickFalse);
+    }
   xmlFreeDoc(document);
   return(MagickTrue);
 }
 #else
-static MagickBooleanType ValidateXMPProfile(const StringInfo *profile)
+static MagickBooleanType ValidateXMPProfile(Image *image,
+  const StringInfo *profile,ExceptionInfo *exception)
 {
+  (void) ThrowMagickException(exception,GetMagickModule(),MissingDelegateError,
+    "DelegateLibrarySupportNotBuiltIn","'%s' (XML)",image->filename);
   return(MagickFalse);
 }
 #endif
@@ -1880,15 +1940,19 @@ static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
   MagickBooleanType
     status;
 
+  StringInfo
+    *clone_profile;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  clone_profile=CloneStringInfo(profile);
+  PatchCorruptProfile(name,clone_profile);
   if ((LocaleCompare(name,"xmp") == 0) &&
-      (ValidateXMPProfile(profile) == MagickFalse))
+      (ValidateXMPProfile(image,clone_profile,exception) == MagickFalse))
     {
-      (void) ThrowMagickException(exception,GetMagickModule(),ImageWarning,
-        "CorruptImageProfile","`%s'",name);
+      clone_profile=DestroyStringInfo(clone_profile);
       return(MagickTrue);
     }
   if (image->profiles == (SplayTreeInfo *) NULL)
@@ -1897,14 +1961,14 @@ static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
   (void) CopyMagickString(key,name,MagickPathExtent);
   LocaleLower(key);
   status=AddValueToSplayTree((SplayTreeInfo *) image->profiles,
-    ConstantString(key),CloneStringInfo(profile));
+    ConstantString(key),clone_profile);
   if (status != MagickFalse)
     {
       if (LocaleCompare(name,"8bim") == 0)
-        GetProfilesFromResourceBlock(image,profile,exception);
+        GetProfilesFromResourceBlock(image,clone_profile,exception);
       else
         if (recursive == MagickFalse)
-          WriteTo8BimProfile(image,name,profile);
+          WriteTo8BimProfile(image,name,clone_profile);
     }
   return(status);
 }
@@ -2385,3 +2449,147 @@ MagickPrivate MagickBooleanType SyncImageProfiles(Image *image)
       status=MagickFalse;
   return(status);
 }
+
+static void UpdateClipPath(unsigned char *blob,size_t length,
+  const size_t old_columns,const size_t old_rows,
+  const RectangleInfo *new_geometry)
+{
+  register ssize_t
+    i;
+
+  ssize_t
+    knot_count,
+    selector;
+
+  knot_count=0;
+  while (length != 0)
+  {
+    selector=(ssize_t) ReadProfileMSBShort(&blob,&length);
+    switch (selector)
+    {
+      case 0:
+      case 3:
+      {
+        if (knot_count != 0)
+          {
+            blob+=24;
+            length-=MagickMin(24,(ssize_t) length);
+            break;
+          }
+        /*
+          Expected subpath length record.
+        */
+        knot_count=(ssize_t) ReadProfileMSBShort(&blob,&length);
+        blob+=22;
+        length-=MagickMin(22,(ssize_t) length);
+        break;
+      }
+      case 1:
+      case 2:
+      case 4:
+      case 5:
+      {
+        if (knot_count == 0)
+          {
+            /*
+              Unexpected subpath knot.
+            */
+            blob+=24;
+            length-=MagickMin(24,(ssize_t) length);
+            break;
+          }
+        /*
+          Add sub-path knot
+        */
+        for (i=0; i < 3; i++)
+        {
+          double
+            x,
+            y;
+
+          signed int
+            xx,
+            yy;
+
+          y=(double) ReadProfileMSBLong(&blob,&length);
+          y=y*old_rows/4096/4096;
+          y-=new_geometry->y;
+          yy=(signed int) ((y*4096*4096)/new_geometry->height);
+          WriteProfileLong(MSBEndian,(size_t) yy,blob-4);
+          x=(double) ReadProfileMSBLong(&blob,&length);
+          x=x*old_columns/4096/4096;
+          x-=new_geometry->x;
+          xx=(signed int) ((x*4096*4096)/new_geometry->width);
+          WriteProfileLong(MSBEndian,(size_t) xx,blob-4);
+        }
+        knot_count--;
+        break;
+      }
+      case 6:
+      case 7:
+      case 8:
+      default:
+      {
+        blob+=24;
+        length-=MagickMin(24,(ssize_t) length);
+        break;
+      }
+    }
+  }
+}
+
+MagickPrivate void Update8BIMClipPath(const Image *image,
+  const size_t old_columns,const size_t old_rows,
+  const RectangleInfo *new_geometry)
+{
+  const StringInfo
+    *profile;
+
+  size_t
+    length;
+
+  ssize_t
+    count,
+    id;
+
+  unsigned char
+    *info;
+
+  assert(image != (Image *) NULL);
+  assert(new_geometry != (RectangleInfo *) NULL);
+  profile=GetImageProfile(image,"8bim");
+  if (profile == (StringInfo *) NULL)
+    return;
+  length=GetStringInfoLength(profile);
+  info=GetStringInfoDatum(profile);
+  while (length > 0)
+  {
+    if (ReadProfileByte(&info,&length) != (unsigned char) '8')
+      continue;
+    if (ReadProfileByte(&info,&length) != (unsigned char) 'B')
+      continue;
+    if (ReadProfileByte(&info,&length) != (unsigned char) 'I')
+      continue;
+    if (ReadProfileByte(&info,&length) != (unsigned char) 'M')
+      continue;
+    id=(ssize_t) ReadProfileMSBShort(&info,&length);
+    count=(ssize_t) ReadProfileByte(&info,&length);
+    if ((count != 0) && ((size_t) count <= length))
+      {
+        info+=count;
+        length-=count;
+      }
+    if ((count & 0x01) == 0)
+      (void) ReadProfileByte(&info,&length);
+    count=(ssize_t) ReadProfileMSBLong(&info,&length);
+    if ((count < 0) || ((size_t) count > length))
+      {
+        length=0;
+        continue;
+      }
+    if ((id > 1999) && (id < 2999))
+      UpdateClipPath(info,(size_t) count,old_columns,old_rows,new_geometry);
+    info+=count;
+    length-=MagickMin(count,(ssize_t) length);
+  }
+}
