@@ -13,17 +13,17 @@
 %                      Render Text Onto A Canvas Image.                       %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,39 +39,41 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/annotate.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/color.h"
-#include "magick/color-private.h"
-#include "magick/colorspace.h"
-#include "magick/constitute.h"
-#include "magick/draw.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/option.h"
-#include "magick/pixel-private.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/statistic.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/annotate.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/color.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/draw.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/statistic.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "coders/txt.h"
 
 /*
   Forward declarations.
 */
 static MagickBooleanType
-  WriteTXTImage(const ImageInfo *,Image *);
+  WriteTXTImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -93,18 +95,15 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
 */
 static MagickBooleanType IsTXT(const unsigned char *magick,const size_t length)
 {
-#define MagickID  "# ImageMagick pixel enumeration:"
-
   char
-    colorspace[MaxTextExtent];
+    colorspace[MagickPathExtent];
 
   ssize_t
     count;
@@ -116,10 +115,11 @@ static MagickBooleanType IsTXT(const unsigned char *magick,const size_t length)
 
   if (length < 40)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,MagickID,strlen(MagickID)) != 0)
+  if (LocaleNCompare((const char *) magick,MagickTXTID,
+        strlen(MagickTXTID)) != 0)
     return(MagickFalse);
-  count=(ssize_t) sscanf((char *) magick+32,"%lu,%lu,%lu,%s",&columns,&rows,
-    &depth,colorspace);
+  count=(ssize_t) sscanf((const char *) magick+32,"%lu,%lu,%lu,%32s",&columns,
+    &rows,&depth,colorspace);
   if (count != 4)
     return(MagickFalse);
   return(MagickTrue);
@@ -142,13 +142,17 @@ static MagickBooleanType IsTXT(const unsigned char *magick,const size_t length)
 %
 %  The format of the ReadTEXTImage method is:
 %
-%      Image *ReadTEXTImage(const ImageInfo *image_info,
-%        ExceptionInfo *exception)
+%      Image *ReadTEXTImage(const ImageInfo *image_info,Image *image,
+%        char *text,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image_info: the image info.
 %
+%    o image: the image.
+%
+%    o text: the text storage buffer.
+%
 %    o exception: return any errors or warnings in this structure.
 %
 */
@@ -156,10 +160,10 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   char
-    filename[MaxTextExtent],
-    geometry[MaxTextExtent],
+    filename[MagickPathExtent],
+    geometry[MagickPathExtent],
     *p,
-    text[MaxTextExtent];
+    text[MagickPathExtent];
 
   DrawInfo
     *draw_info;
@@ -168,9 +172,6 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
     *image,
     *texture;
 
-  long
-    offset;
-
   MagickBooleanType
     status;
 
@@ -180,6 +181,9 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   RectangleInfo
     page;
 
+  ssize_t
+    offset;
+
   TypeMetric
     metrics;
 
@@ -187,25 +191,27 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  (void) memset(text,0,sizeof(text));
+  (void) ReadBlobString(image,text);
   /*
     Set the page geometry.
   */
   delta.x=DefaultResolution;
   delta.y=DefaultResolution;
-  if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
+  if ((image->resolution.x == 0.0) || (image->resolution.y == 0.0))
     {
       GeometryInfo
         geometry_info;
@@ -214,10 +220,10 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
         flags;
 
       flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-      image->x_resolution=geometry_info.rho;
-      image->y_resolution=geometry_info.sigma;
+      image->resolution.x=geometry_info.rho;
+      image->resolution.y=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
-        image->y_resolution=image->x_resolution;
+        image->resolution.y=image->resolution.x;
     }
   page.width=612;
   page.height=792;
@@ -228,16 +234,17 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   /*
     Initialize Image structure.
   */
-  image->columns=(unsigned long) (((page.width*image->x_resolution)/
+  image->columns=(size_t) floor((((double) page.width*image->resolution.x)/
     delta.x)+0.5);
-  image->rows=(unsigned long) (((page.height*image->y_resolution)/delta.y)+0.5);
+  image->rows=(size_t) floor((((double) page.height*image->resolution.y)/
+    delta.y)+0.5);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status != MagickFalse)
+    status=ResetImagePixels(image,exception);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   image->page.x=0;
   image->page.y=0;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   texture=(Image *) NULL;
   if (image_info->texture != (char *) NULL)
     {
@@ -247,28 +254,33 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
       read_info=CloneImageInfo(image_info);
       SetImageInfoBlob(read_info,(void *) NULL,0);
       (void) CopyMagickString(read_info->filename,image_info->texture,
-        MaxTextExtent);
+        MagickPathExtent);
       texture=ReadImage(read_info,exception);
       read_info=DestroyImageInfo(read_info);
     }
   /*
     Annotate the text image.
   */
-  (void) SetImageBackgroundColor(image);
+  (void) SetImageBackgroundColor(image,exception);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   (void) CloneString(&draw_info->text,image_info->filename);
-  (void) FormatMagickString(geometry,MaxTextExtent,"0x0%+ld%+ld",page.x,page.y);
+  (void) FormatLocaleString(geometry,MagickPathExtent,"%gx%g%+g%+g",(double)
+    image->columns,(double) image->rows,(double) page.x,(double) page.y);
   (void) CloneString(&draw_info->geometry,geometry);
-  status=GetTypeMetrics(image,draw_info,&metrics);
+  status=GetTypeMetrics(image,draw_info,&metrics,exception);
   if (status == MagickFalse)
-    ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
-  page.y=(long) (page.y+metrics.ascent+0.5);
-  (void) FormatMagickString(geometry,MaxTextExtent,"0x0%+ld%+ld",page.x,page.y);
+    {
+      draw_info=DestroyDrawInfo(draw_info);
+      ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
+    }
+  page.y=(ssize_t) ceil((double) page.y+metrics.ascent-0.5);
+  (void) FormatLocaleString(geometry,MagickPathExtent,"%gx%g%+g%+g",(double)
+    image->columns,(double) image->rows,(double) page.x,(double) page.y);
   (void) CloneString(&draw_info->geometry,geometry);
-  (void) CopyMagickString(filename,image_info->filename,MaxTextExtent);
+  (void) CopyMagickString(filename,image_info->filename,MagickPathExtent);
   if (*draw_info->text != '\0')
     *draw_info->text='\0';
-  p=ReadBlobString(image,text);
+  p=text;
   for (offset=2*page.y; p != (char *) NULL; )
   {
     /*
@@ -276,15 +288,16 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
     */
     (void) ConcatenateString(&draw_info->text,text);
     (void) ConcatenateString(&draw_info->text,"\n");
-    offset+=(long) (metrics.ascent-metrics.descent);
+    offset+=(ssize_t) (metrics.ascent-metrics.descent);
     if (image->previous == (Image *) NULL)
       {
-        status=SetImageProgress(image,LoadImageTag,offset,image->rows);
+        status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) offset,
+          image->rows);
         if (status == MagickFalse)
           break;
       }
     p=ReadBlobString(image,text);
-    if ((offset < (long) image->rows) && (p != (char *) NULL))
+    if ((offset < (ssize_t) image->rows) && (p != (char *) NULL))
       continue;
     if (texture != (Image *) NULL)
       {
@@ -293,11 +306,11 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
 
         progress_monitor=SetImageProgressMonitor(image,
           (MagickProgressMonitor) NULL,image->client_data);
-        (void) TextureImage(image,texture);
+        (void) TextureImage(image,texture,exception);
         (void) SetImageProgressMonitor(image,progress_monitor,
           image->client_data);
       }
-    (void) AnnotateImage(image,draw_info);
+    (void) AnnotateImage(image,draw_info,exception);
     if (p == (char *) NULL)
       break;
     /*
@@ -305,17 +318,17 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
     */
     *draw_info->text='\0';
     offset=2*page.y;
-    AcquireNextImage(image_info,image);
+    AcquireNextImage(image_info,image,exception);
     if (GetNextImageInList(image) == (Image *) NULL)
       {
-        image=DestroyImageList(image);
-        return((Image *) NULL);
+        status=MagickFalse;
+        break;
       }
     image->next->columns=image->columns;
     image->next->rows=image->rows;
     image=SyncNextImageInList(image);
-    (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-    (void) SetImageBackgroundColor(image);
+    (void) CopyMagickString(image->filename,filename,MagickPathExtent);
+    (void) SetImageBackgroundColor(image,exception);
     status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
       GetBlobSize(image));
     if (status == MagickFalse)
@@ -328,14 +341,16 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
 
       progress_monitor=SetImageProgressMonitor(image,
         (MagickProgressMonitor) NULL,image->client_data);
-      (void) TextureImage(image,texture);
+      (void) TextureImage(image,texture,exception);
       (void) SetImageProgressMonitor(image,progress_monitor,image->client_data);
     }
-  (void) AnnotateImage(image,draw_info);
+  (void) AnnotateImage(image,draw_info,exception);
   if (texture != (Image *) NULL)
     texture=DestroyImage(texture);
   draw_info=DestroyDrawInfo(draw_info);
   (void) CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -368,122 +383,237 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
 static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
-    colorspace[MaxTextExtent],
-    text[MaxTextExtent];
+    colorspace[MagickPathExtent],
+    text[MagickPathExtent];
+
+  double
+    max_value,
+    x_offset,
+    y_offset;
 
   Image
     *image;
 
-  IndexPacket
-    *indexes;
-
-  long
-    type,
-    x,
-    y;
-
-  LongPixelPacket
+  PixelInfo
     pixel;
 
   MagickBooleanType
     status;
 
   QuantumAny
-    scale;
+    range;
 
-  register long
-    i;
+  register ssize_t
+    i,
+    x;
 
-  register PixelPacket
+  register Quantum
     *q;
 
   ssize_t
-    count;
+    count,
+    type,
+    y;
 
   unsigned long
     depth,
-    max_value;
+    height,
+    width;
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  (void) ResetMagickMemory(text,0,sizeof(text));
+  (void) memset(text,0,sizeof(text));
   (void) ReadBlobString(image,text);
-  if (LocaleNCompare((char *) text,MagickID,strlen(MagickID)) != 0)
+  if (LocaleNCompare((char *) text,MagickTXTID,strlen(MagickTXTID)) != 0)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  *colorspace='\0';
-  count=(ssize_t) sscanf(text+32,"%lu,%lu,%lu,%s",&image->columns,
-    &image->rows,&max_value,colorspace);
-  if (count != 4)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  for (depth=1; (GetQuantumRange(depth)+1) < max_value; depth++) ;
-  image->depth=depth;
-  LocaleLower(colorspace);
-  i=(long) strlen(colorspace)-1;
-  image->matte=MagickFalse;
-  if ((i > 0) && (colorspace[i] == 'a'))
-    {
-      colorspace[i]='\0';
-      image->matte=MagickTrue;
-    }
-  type=ParseMagickOption(MagickColorspaceOptions,MagickFalse,colorspace);
-  if (type < 0)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  image->colorspace=(ColorspaceType) type;
-  (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
-  (void) SetImageBackgroundColor(image);
-  scale=GetQuantumScale(image->depth);
-  while (ReadBlobString(image,text) != (char *) NULL)
+  x_offset=(-1.0);
+  y_offset=(-1.0);
+  q=(Quantum *) NULL;
+  do
   {
-    if (image->colorspace == CMYKColorspace)
+    width=0;
+    height=0;
+    max_value=0.0;
+    *colorspace='\0';
+    count=(ssize_t) sscanf(text+32,"%lu,%lu,%lf,%32s",&width,&height,&max_value,
+      colorspace);
+    if ((count != 4) || (width == 0) || (height == 0) || (max_value == 0.0))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    image->columns=width;
+    image->rows=height;
+    if ((max_value == 0.0) || (max_value > 18446744073709551615.0))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    for (depth=1; (GetQuantumRange(depth)+1.0) < max_value; depth++) ;
+    image->depth=depth;
+    status=SetImageExtent(image,image->columns,image->rows,exception);
+    if (status != MagickFalse)
+      status=ResetImagePixels(image,exception);
+    if (status == MagickFalse)
+      return(DestroyImageList(image));
+    LocaleLower(colorspace);
+    i=(ssize_t) strlen(colorspace)-1;
+    image->alpha_trait=UndefinedPixelTrait;
+    if ((i > 0) && (colorspace[i] == 'a'))
       {
-        if (image->matte != MagickFalse)
-          count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu,%lu",&x,&y,
-            &pixel.red,&pixel.green,&pixel.blue,&pixel.index,&pixel.opacity);
-        else
-          count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu",&x,&y,
-            &pixel.red,&pixel.green,&pixel.blue,&pixel.index);
+        colorspace[i]='\0';
+        image->alpha_trait=BlendPixelTrait;
       }
-    else
-      if (image->matte != MagickFalse)
-        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu",&x,&y,
-          &pixel.red,&pixel.green,&pixel.blue,&pixel.opacity);
-      else
-        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu",&x,&y,
-          &pixel.red,&pixel.green,&pixel.blue);
-    if (count < 5)
-      continue;
-    q=GetAuthenticPixels(image,x,y,1,1,exception);
-    if (q == (PixelPacket *) NULL)
-      continue;
-    q->red=ScaleAnyToQuantum(pixel.red,image->depth,scale);
-    q->green=ScaleAnyToQuantum(pixel.green,image->depth,scale);
-    q->blue=ScaleAnyToQuantum(pixel.blue,image->depth,scale);
-    if (image->colorspace == CMYKColorspace)
+    type=ParseCommandOption(MagickColorspaceOptions,MagickFalse,colorspace);
+    if (type < 0)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    (void) SetImageColorspace(image,(ColorspaceType) type,exception);
+    (void) SetImageBackgroundColor(image,exception);
+    GetPixelInfo(image,&pixel);
+    range=GetQuantumRange(image->depth);
+    status=MagickTrue;
+    for (y=0; y < (ssize_t) image->rows; y++)
+    {
+      double
+        alpha,
+        black,
+        blue,
+        green,
+        red;
+
+      if (status == MagickFalse)
+        break;
+      red=0.0;
+      green=0.0;
+      blue=0.0;
+      black=0.0;
+      alpha=0.0;
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
-        indexes=GetAuthenticIndexQueue(image);
-        *indexes=ScaleAnyToQuantum(pixel.index,image->depth,scale);
+        if (ReadBlobString(image,text) == (char *) NULL)
+          {
+            status=MagickFalse;
+            break;
+          }
+        switch (image->colorspace)
+        {
+          case LinearGRAYColorspace:
+          case GRAYColorspace:
+          {
+            if (image->alpha_trait != UndefinedPixelTrait)
+              {
+                count=(ssize_t) sscanf(text,"%lf,%lf: (%lf%*[%,]%lf%*[%,]",
+                  &x_offset,&y_offset,&red,&alpha);
+                green=red;
+                blue=red;
+                break;
+              }
+            count=(ssize_t) sscanf(text,"%lf,%lf: (%lf%*[%,]",&x_offset,
+              &y_offset,&red);
+            green=red;
+            blue=red;
+            break;
+          }
+          case CMYKColorspace:
+          {
+            if (image->alpha_trait != UndefinedPixelTrait)
+              {
+                count=(ssize_t) sscanf(text,
+                  "%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]",
+                  &x_offset,&y_offset,&red,&green,&blue,&black,&alpha);
+                break;
+              }
+            count=(ssize_t) sscanf(text,
+              "%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]",&x_offset,
+              &y_offset,&red,&green,&blue,&black);
+            break;
+          }
+          default:
+          {
+            if (image->alpha_trait != UndefinedPixelTrait)
+              {
+                count=(ssize_t) sscanf(text,
+                  "%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]",
+                  &x_offset,&y_offset,&red,&green,&blue,&alpha);
+                break;
+              }
+            count=(ssize_t) sscanf(text,"%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]",
+              &x_offset,&y_offset,&red,&green,&blue);
+            break;
+          }
+        }
+        if (strchr(text,'%') != (char *) NULL)
+          {
+            red*=0.01*range;
+            green*=0.01*range;
+            blue*=0.01*range;
+            black*=0.01*range;
+            alpha*=0.01*range;
+          }
+        if (image->colorspace == LabColorspace)
+          {
+            green+=(range+1)/2.0;
+            blue+=(range+1)/2.0;
+          }
+        pixel.red=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
+          MagickMax(red+0.5,0.0),range);
+        pixel.green=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
+          MagickMax(green+0.5,0.0),range);
+        pixel.blue=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
+          MagickMax(blue+0.5,0.0),range);
+        pixel.black=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
+          MagickMax(black+0.5,0.0),range);
+        pixel.alpha=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
+          MagickMax(alpha+0.5,0.0),range);
+        q=GetAuthenticPixels(image,(ssize_t) x_offset,(ssize_t) y_offset,1,1,
+          exception);
+        if (q == (Quantum *) NULL)
+          {
+            status=MagickFalse;
+            break;
+          }
+        SetPixelViaPixelInfo(image,&pixel,q);
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
+          {
+            status=MagickFalse;
+            break;
+          }
       }
-    if (image->matte != MagickFalse)
-      q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel.opacity,
-        image->depth,scale));
-    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+    }
+    if (status == MagickFalse)
       break;
-  }
+    *text='\0';
+    (void) ReadBlobString(image,text);
+    if (LocaleNCompare((char *) text,MagickTXTID,strlen(MagickTXTID)) == 0)
+      {
+        /*
+          Allocate next image structure.
+        */
+        AcquireNextImage(image_info,image,exception);
+        if (GetNextImageInList(image) == (Image *) NULL)
+          {
+            status=MagickFalse;
+            break;
+          }
+        image=SyncNextImageInList(image);
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
+      }
+  } while (LocaleNCompare((char *) text,MagickTXTID,strlen(MagickTXTID)) == 0);
+  (void) CloseBlob(image);
+  if (q == (Quantum *) NULL)
+    return(DestroyImage(image));
   return(GetFirstImageInList(image));
 }
 
@@ -507,30 +637,29 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterTXTImage method is:
 %
-%      unsigned long RegisterTXTImage(void)
+%      size_t RegisterTXTImage(void)
 %
 */
-ModuleExport unsigned long RegisterTXTImage(void)
+ModuleExport size_t RegisterTXTImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("TEXT");
-  entry->decoder=(DecodeImageHandler *) ReadTEXTImage;
+  entry=AcquireMagickInfo("TXT","SPARSE-COLOR","Sparse Color");
   entry->encoder=(EncodeImageHandler *) WriteTXTImage;
-  entry->raw=MagickTrue;
-  entry->adjoin=MagickFalse;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Text");
-  entry->module=ConstantString("TXT");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("TXT");
+  entry=AcquireMagickInfo("TXT","TEXT","Text");
+  entry->decoder=(DecodeImageHandler *) ReadTEXTImage;
+  entry->format_type=ImplicitFormatType;
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  (void) RegisterMagickInfo(entry);
+  entry=AcquireMagickInfo("TXT","TXT","Text");
   entry->decoder=(DecodeImageHandler *) ReadTXTImage;
   entry->encoder=(EncodeImageHandler *) WriteTXTImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Text");
   entry->magick=(IsImageFormatHandler *) IsTXT;
-  entry->module=ConstantString("TXT");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -556,6 +685,7 @@ ModuleExport unsigned long RegisterTXTImage(void)
 */
 ModuleExport void UnregisterTXTImage(void)
 {
+  (void) UnregisterMagickInfo("SPARSE-COLOR");
   (void) UnregisterMagickInfo("TEXT");
   (void) UnregisterMagickInfo("TXT");
 }
@@ -575,7 +705,8 @@ ModuleExport void UnregisterTXTImage(void)
 %
 %  The format of the WriteTXTImage method is:
 %
-%      MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteTXTImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -583,99 +714,158 @@ ModuleExport void UnregisterTXTImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   char
-    buffer[MaxTextExtent],
-    colorspace[MaxTextExtent],
-    tuple[MaxTextExtent];
-
-  long
-    y;
+    buffer[MagickPathExtent],
+    colorspace[MagickPathExtent],
+    tuple[MagickPathExtent];
 
   MagickBooleanType
     status;
 
-  MagickPixelPacket
+  MagickOffsetType
+    scene;
+
+  PixelInfo
     pixel;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
+  register const Quantum
     *p;
 
-  register long
+  register ssize_t
     x;
 
+  size_t
+    imageListLength;
+
+  ssize_t
+    y;
+
   /*
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBlobMode,&image->exception);
+  status=OpenBlob(image_info,image,WriteBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  (void) CopyMagickString(colorspace,MagickOptionToMnemonic(
-    MagickColorspaceOptions,(long) image->colorspace),MaxTextExtent);
-  LocaleLower(colorspace);
-  image->depth=GetImageQuantumDepth(image,MagickTrue);
-  if (image->matte != MagickFalse)
-    (void) ConcatenateMagickString(colorspace,"a",MaxTextExtent);
-  (void) FormatMagickString(buffer,MaxTextExtent,
-    "# ImageMagick pixel enumeration: %lu,%lu,%lu,%s\n",image->columns,
-    image->rows,(unsigned long) GetQuantumRange(image->depth),colorspace);
-  (void) WriteBlobString(image,buffer);
-  GetMagickPixelPacket(image,&pixel);
-  for (y=0; y < (long) image->rows; y++)
+  scene=0;
+  imageListLength=GetImageListLength(image);
+  do
   {
-    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
-      break;
-    indexes=GetVirtualIndexQueue(image);
-    for (x=0; x < (long) image->columns; x++)
+    ComplianceType
+      compliance;
+
+    const char
+      *value;
+
+    (void) CopyMagickString(colorspace,CommandOptionToMnemonic(
+      MagickColorspaceOptions,(ssize_t) image->colorspace),MagickPathExtent);
+    LocaleLower(colorspace);
+    image->depth=GetImageQuantumDepth(image,MagickTrue);
+    if (image->alpha_trait != UndefinedPixelTrait)
+      (void) ConcatenateMagickString(colorspace,"a",MagickPathExtent);
+    compliance=NoCompliance;
+    value=GetImageOption(image_info,"txt:compliance");
+    if (value != (char *) NULL)
+      compliance=(ComplianceType) ParseCommandOption(MagickComplianceOptions,
+        MagickFalse,value);
+    if (LocaleCompare(image_info->magick,"SPARSE-COLOR") != 0)
+      {
+        (void) FormatLocaleString(buffer,MagickPathExtent,
+          "# ImageMagick pixel enumeration: %.20g,%.20g,%.20g,%s\n",(double)
+          image->columns,(double) image->rows,(double)
+          GetQuantumRange(image->depth),colorspace);
+        (void) WriteBlobString(image,buffer);
+      }
+    GetPixelInfo(image,&pixel);
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      (void) FormatMagickString(buffer,MaxTextExtent,"%ld,%ld: ",x,y);
-      (void) WriteBlobString(image,buffer);
-      SetMagickPixelPacket(image,p,indexes+x,&pixel);
-      (void) CopyMagickString(tuple,"(",MaxTextExtent);
-      ConcatenateColorComponent(&pixel,RedChannel,X11Compliance,tuple);
-      (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-      ConcatenateColorComponent(&pixel,GreenChannel,X11Compliance,tuple);
-      (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-      ConcatenateColorComponent(&pixel,BlueChannel,X11Compliance,tuple);
-      if (pixel.colorspace == CMYKColorspace)
-        {
-          (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-          ConcatenateColorComponent(&pixel,IndexChannel,X11Compliance,tuple);
-        }
-      if (pixel.matte != MagickFalse)
-        {
-          (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-          ConcatenateColorComponent(&pixel,AlphaChannel,X11Compliance,tuple);
-        }
-      (void) ConcatenateMagickString(tuple,")",MaxTextExtent);
-      (void) WriteBlobString(image,tuple);
-      (void) WriteBlobString(image,"  ");
-      GetColorTuple(&pixel,MagickTrue,tuple);
-      (void) FormatMagickString(buffer,MaxTextExtent,"%s",tuple);
-      (void) WriteBlobString(image,buffer);
-      (void) WriteBlobString(image,"  ");
-      (void) QueryMagickColorname(image,&pixel,SVGCompliance,tuple,
-        &image->exception);
-      (void) WriteBlobString(image,tuple);
-      (void) WriteBlobString(image,"\n");
-      p++;
+      p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+      if (p == (const Quantum *) NULL)
+        break;
+      for (x=0; x < (ssize_t) image->columns; x++)
+      {
+        GetPixelInfoPixel(image,p,&pixel);
+        if (LocaleCompare(image_info->magick,"SPARSE-COLOR") == 0)
+          {
+            /*
+              Sparse-color format.
+            */
+            if (GetPixelAlpha(image,p) == (Quantum) OpaqueAlpha)
+              {
+                GetColorTuple(&pixel,MagickFalse,tuple);
+                (void) FormatLocaleString(buffer,MagickPathExtent,
+                  "%.20g,%.20g,",(double) x,(double) y);
+                (void) WriteBlobString(image,buffer);
+                (void) WriteBlobString(image,tuple);
+                (void) WriteBlobString(image," ");
+              }
+            p+=GetPixelChannels(image);
+            continue;
+          }
+        (void) FormatLocaleString(buffer,MagickPathExtent,"%.20g,%.20g: ",
+          (double) x,(double) y);
+        (void) WriteBlobString(image,buffer);
+        (void) CopyMagickString(tuple,"(",MagickPathExtent);
+        if ((pixel.colorspace == LinearGRAYColorspace) ||
+            (pixel.colorspace == GRAYColorspace))
+          ConcatenateColorComponent(&pixel,GrayPixelChannel,compliance,tuple);
+        else
+          {
+            ConcatenateColorComponent(&pixel,RedPixelChannel,compliance,tuple);
+            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
+            ConcatenateColorComponent(&pixel,GreenPixelChannel,compliance,
+              tuple);
+            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
+            ConcatenateColorComponent(&pixel,BluePixelChannel,compliance,
+              tuple);
+          }
+        if (pixel.colorspace == CMYKColorspace)
+          {
+            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
+            ConcatenateColorComponent(&pixel,BlackPixelChannel,compliance,
+              tuple);
+          }
+        if (pixel.alpha_trait != UndefinedPixelTrait)
+          {
+            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
+            ConcatenateColorComponent(&pixel,AlphaPixelChannel,compliance,
+              tuple);
+          }
+        (void) ConcatenateMagickString(tuple,")",MagickPathExtent);
+        (void) WriteBlobString(image,tuple);
+        (void) WriteBlobString(image,"  ");
+        GetColorTuple(&pixel,MagickTrue,tuple);
+        (void) FormatLocaleString(buffer,MagickPathExtent,"%s",tuple);
+        (void) WriteBlobString(image,buffer);
+        (void) WriteBlobString(image,"  ");
+        (void) QueryColorname(image,&pixel,SVGCompliance,tuple,exception);
+        (void) WriteBlobString(image,tuple);
+        (void) WriteBlobString(image,"\n");
+        p+=GetPixelChannels(image);
+      }
+      status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+        image->rows);
+      if (status == MagickFalse)
+        break;
     }
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (GetNextImageInList(image) == (Image *) NULL)
+      break;
+    image=SyncNextImageInList(image);
+    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
     if (status == MagickFalse)
       break;
-  }
+  } while (image_info->adjoin != MagickFalse);
   (void) CloseBlob(image);
   return(MagickTrue);
 }
